#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <net/ethernet.h>
#include <sys/time.h>
#include <netinet/ip.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <mapi.h>
#include <string.h>

// declare some functions
static void terminate();

// globally initialize the mapi-flows, fd and st 
int fd, st;

// global packet counters, fid, sit
int fid = 0, sit = 0;

// global packet counter
mapi_results_t *dres, *sres;
unsigned long long *total_packets;

// time variables
time_t start, end;


// program starts here 
int main(int argc, char *argv[]) {
	

	// prgram description and help
   if( (strcmp (argv[1], "--help") == 0) || (strcmp (argv[1], "-h") == 0) ){
		printf("Usage: IDS -b \"BPF-Filter\" -s \"str search\"  <file>\n");
		printf(" -b \"BPF-filter\"  is, if specified, a BPF filter to apply the packet capture (optional)\n");
		printf(" -s \"str_search\"  is, if specified, a string to search for the packet paylod (optional)\n");	
		printf(" <file>		  The outputfile where the trace is to be stored (required) \n");
		printf("\n The quotes around BPF-filter and str_search are required!\n");
		return -1;	
   }

	// check if the number of arguments given are correct
	if(!(argc==2 || argc == 4 || argc==6)){
	   printf("IDS: wrong arguments \n");
		printf("Try `IDS --help` for more information.\n");
		return -1;
   }

	// define signals which calls terminate() and aborts the program; e.g, ctrl-c 
	signal(SIGINT, terminate);
	signal(SIGQUIT, terminate);
	signal(SIGTERM, terminate);


	// create a flow using the dag0 interface
	fd = mapi_create_flow ("/dev/dag0");

	// create another flow using the dag0 interface for total statistics
	st = mapi_create_flow ("/dev/dag0");


	// check whether we could create flow  fd
	if( fd < 0) {
   	printf("Could not create flow %d\n", fd);
   	exit(EXIT_FAILURE);
	 }
	
	// check whether we could create flow  st
	if (st <0) {
      printf("Could not create flow %d\n", st);
      exit(EXIT_FAILURE);
    }



	// writing parameter info
	printf("\n==== Parameter info ===\n");

	// apply BPF-filter if it's specified
	if ( (argc==4 || argc == 6) && (strcmp (argv[1], "-b") == 0) ){
		printf("BPF filter: Found!\n   Applying to flow: '%s' \n", argv[2]);
		mapi_apply_function (fd, "BPF_FILTER", argv[2] ); 
	}
	else{
		printf("BPF filter: Not found!\n   Ignoring..\n");
	}


   // apply str search if it's specified
   if ( (argc==4 || argc == 6) && ( (strcmp (argv[1], "-s") == 0) || (strcmp (argv[3], "-s") == 0) ) ){
      printf("String search: Found!\n   Applying to flow '");
		if (strcmp (argv[1], "-s") == 0){ 
			printf("%s'\n", argv[2]);
			//mapi_apply_function (fd, "STR_SEARCH", "bot-testen-her", 0, 1500);
			mapi_apply_function (fd, "STR_SEARCH", argv[2], 0, 1500);
			//mapi_apply_function (fd, "STR_SEARCH", "bot-testen-her", 0, 1500);
		}
		else if (strcmp (argv[3], "-s") == 0){
			printf("%s'\n", argv[4]);
			//mapi_apply_function (fd, "STR_SEARCH", "bot-testen-her", 0, 1500);
			mapi_apply_function (fd, "STR_SEARCH", argv[4], 0, 1500 );
			//mapi_apply_function (fd, "STR_SEARCH", "bot-testen-her", 0, 1500);
		}
   }
   else{
      printf("String search: Not found!\n   Ignoring..\n");
   }

	// check if a file argument is given
	if ( argc == 2 || argc==4 || argc == 6){
		if ( (argc == 2) && (strlen(argv[1]) > 0) ) {
			printf("File: Found!\n   Writing to %s\n", argv[1]);
			mapi_apply_function (fd, "TO_FILE", MFF_PCAP, argv[1], 0);
		}
		else if ( (argc == 4) && (strlen(argv[3]) > 0) ){
			printf("File: Found!\n   Writing to %s\n", argv[3]);
			mapi_apply_function (fd, "TO_FILE", MFF_PCAP, argv[3], 0);
		}
		else if ( (argc == 6) && (strlen(argv[5]) > 0) ){
			printf("File: Found!\n   Writing to %s\n", argv[5]);
			mapi_apply_function (fd, "TO_FILE", MFF_PCAP, argv[5], 0);
		}
		else {
			printf("File: Not found!\n   Aborting!\n");
			return -1;
		}
	}
	else{
         printf("File: Not found!\n   Aborting!\n");
         return -1;
	}

   // packet couting on the st flow
   sit = mapi_apply_function(st,"PKT_COUNTER");

   // packet couting on the fd flow
   fid = mapi_apply_function(fd,"PKT_COUNTER");



	// start the fd flow
   if(mapi_connect(fd) < 0) {
       printf("Could not connect to flow %d\n", fd);
       exit(EXIT_FAILURE);
   }
	
	// start the st flow
	if(mapi_connect(st) < 0) {
		 printf("Could not connect to flow %d\n", st);
		 exit(EXIT_FAILURE);
	}
	
	// start script	
	printf ("\n Working...\n");

	// start the timer
	time(&start);
	
	


	// make the script run forever
	while(1){
		sleep(1);
	}
	
 return 0; 
}

void terminate() {

	time(&end);
	double dif = difftime (end,start);

	printf ("\n Finished..\n");

	printf ("\n=== Statistics ===\n");
	printf ("Run-time: %.2lf seconds.\n\n", dif );
	
	sres = mapi_read_results(st, sit);
	dres = mapi_read_results(fd, fid);
   printf("Total pkts : %llu\n", *((unsigned long long*)sres->res));
	printf("Matched packets : %llu\n", *((unsigned long long*)dres->res));

	//printf("Total packets: %lld\n", *total_packets);

	mapi_close_flow(fd);
	mapi_close_flow(st);
	exit(EXIT_SUCCESS);
}

